@@ -92,7 +92,7 @@ def main(args=None):
         os.environ["KRB5_KTNAME"] = conf.get("kerberos", "keytab")
 
     import argcomplete
-    from dbnd_airflow.scheduler.zombies import find_and_kill_dagrun_zombies
+    from dbnd_airflow.scheduler.dagrun_zombies import find_and_kill_dagrun_zombies
 
     CLIFactory.subparsers_dict[find_and_kill_dagrun_zombies.__name__] = {
         "func": find_and_kill_dagrun_zombies,
