@@ -19,17 +19,21 @@
 import time
 import typing
 
-from typing import Dict
+from typing import Dict, Optional
 
 import attr
 
+from airflow.utils.dag_processing import SimpleTaskInstance
 from airflow.utils.db import provide_session
 from airflow.utils.state import State
 from airflow.utils.timezone import utcnow
 
 from dbnd._core.constants import TaskRunState
 from dbnd._core.current import try_get_databand_run
-from dbnd._core.errors.friendly_error.executor_k8s import KubernetesImageNotFoundError
+from dbnd._core.errors.friendly_error.executor_k8s import (
+    KubernetesImageNotFoundError,
+    KubernetesPodConfigFailure,
+)
 from dbnd._core.log.logging_utils import PrefixLoggerAdapter
 from dbnd._core.task_run.task_run_error import TaskRunError
 from dbnd_airflow.airflow_extensions.dal import (
@@ -47,6 +51,7 @@ from dbnd_airflow.executors.kubernetes_executor.kubernetes_watcher import (
 )
 from dbnd_airflow.executors.kubernetes_executor.utils import mgr_init
 from dbnd_airflow_contrib.kubernetes_metrics_logger import KubernetesMetricsLogger
+from dbnd_docker.kubernetes.dns1123_clean_names import create_pod_id
 from dbnd_docker.kubernetes.kube_dbnd_client import (
     PodFailureReason,
     _get_status_log_safe,
@@ -56,6 +61,9 @@ from dbnd_docker.kubernetes.kube_dbnd_client import (
 
 if typing.TYPE_CHECKING:
     from dbnd_docker.kubernetes.kubernetes_engine_config import KubernetesEngineConfig
+    from dbnd._core.task_run.task_run import TaskRun
+    from kubernetes.client import V1Pod
+    from sqlalchemy.orm import Session
 
 
 @attr.s
@@ -63,7 +71,7 @@ class SubmittedPodState(object):
     pod_name = attr.ib()
     submitted_at = attr.ib()
 
-    # key from airflow (task instance
+    # task_instance key: dag_id, task_id, execution_date, try_number
     scheduler_key = attr.ib()
     # dbnd run
     task_run = attr.ib()
@@ -80,11 +88,34 @@ class SubmittedPodState(object):
     def try_number(self):
         return self.scheduler_key[1]
 
+    @property
+    def is_started_running(self):
+        # the node name is set when the pod started running
+        return self.node_name is not None
+
+
+AIRFLOW_TO_DBND_STATE_MAP = {
+    State.SUCCESS: TaskRunState.SUCCESS,
+    State.UP_FOR_RETRY: TaskRunState.UP_FOR_RETRY,
+    State.UP_FOR_RESCHEDULE: TaskRunState.UP_FOR_RETRY,
+    State.FAILED: TaskRunState.FAILED,
+    State.SHUTDOWN: TaskRunState.FAILED,
+    State.SKIPPED: TaskRunState.SKIPPED,
+    State.QUEUED: TaskRunState.QUEUED,
+    State.RUNNING: TaskRunState.RUNNING,
+    State.UPSTREAM_FAILED: TaskRunState.UPSTREAM_FAILED,
+}
+
 
 class DbndKubernetesScheduler(AirflowKubernetesScheduler):
     """
+    The scheduler is responsible for:
+        * running the pods on kubernetes
+        * getting pods' events from the watcher (DbndKubernetesJobWatcher)
+        * handling the state of the task relative to the state of the pods
+
     Very serious override of AirflowKubernetesScheduler
-    1. better visability on errors, so we proceed Failures with much more info
+    1. better visibility on errors, so we proceed Failures with much more info
     2. tracking of all around "airflow run" events -> Pod Crashes, Pod Submission errors
         a. in case of crash (OOM, evicted pod) -> error propogation to databand and retry
     """
@@ -111,7 +142,7 @@ class DbndKubernetesScheduler(AirflowKubernetesScheduler):
         self.current_resource_version = 0
         self.kube_watcher = self._make_kube_watcher_dbnd()
 
-        # pod to airflow key (dag_id, task_id, execution_date)
+        # pod_id to SubmittedPodState
         self.submitted_pods = {}  # type: Dict[str,SubmittedPodState]
 
         # sending data to databand tracker
@@ -141,7 +172,7 @@ class DbndKubernetesScheduler(AirflowKubernetesScheduler):
     @staticmethod
     def _create_pod_id(dag_id, task_id):
         task_run = try_get_databand_run().get_task_run(task_id)
-        return task_run.job_id__dns1123
+        return create_pod_id(task_run)
 
     def _health_check_kube_watcher(self):
         if self.kube_watcher.is_alive():
@@ -324,9 +355,7 @@ class DbndKubernetesScheduler(AirflowKubernetesScheduler):
         if submitted_pod.processed:
             # we already processed this kind of event, as in this process we have failed status already
             self.log.info(
-                "%s Skipping pod '%s' event from %s - already processed",
-                state,
-                pod_name,
+                "Skipping pod '%s' event from %s - already processed", state, pod_name,
             )
             return
 
@@ -366,7 +395,7 @@ class DbndKubernetesScheduler(AirflowKubernetesScheduler):
             )
             return
 
-        pod_data = self.get_pod_status(pod_name)
+        pod_data = self.kube_dbnd.get_pod_status(pod_name)
         if not pod_data or not pod_data.spec.node_name:
             self.log.error("%s: Failed to find pod data for %s", pod_name)
             node_name = "failed_to_find"
@@ -378,6 +407,8 @@ class DbndKubernetesScheduler(AirflowKubernetesScheduler):
         task_run.set_task_run_state(TaskRunState.RUNNING, track=False)
 
     def _process_pod_success(self, submitted_pod):
+        # type: (SubmittedPodState) -> None
+
         task_run = submitted_pod.task_run
         pod_name = submitted_pod.pod_name
 
@@ -421,6 +452,8 @@ class DbndKubernetesScheduler(AirflowKubernetesScheduler):
         )
 
     def _process_pod_failed(self, submitted_pod):
+        # type: (SubmittedPodState) -> None
+
         task_run = submitted_pod.task_run
         pod_name = submitted_pod.pod_name
 
@@ -431,7 +464,7 @@ class DbndKubernetesScheduler(AirflowKubernetesScheduler):
             "%s: pod %s has crashed, airflow state: %s", task_run, pod_name, ti_state
         )
 
-        pod_data = self.get_pod_status(pod_name)
+        pod_data = self.kube_dbnd.get_pod_status(pod_name)
         pod_ctrl = self.kube_dbnd.get_pod_ctrl(pod_name, self.namespace)
 
         pod_logs = []
@@ -449,10 +482,10 @@ class DbndKubernetesScheduler(AirflowKubernetesScheduler):
             ti_state,
         )
         failure_reason, failure_message = self._find_pod_failure_reason(
-            task_run=task_run, pod_data=pod_data, pod_name=pod_name
+            pod_data=pod_data, pod_name=pod_name
         )
         if failure_reason:
-            error_msg += "Found reason for failure: %s - %s." % (
+            error_msg += " Discovered reason for failure is %s: %s." % (
                 failure_reason,
                 failure_message,
             )
@@ -468,25 +501,33 @@ class DbndKubernetesScheduler(AirflowKubernetesScheduler):
             task_run=task_run, msg=error_msg, help_msg=error_help_msg,
         )
 
-        if ti_state == State.FAILED:
-            # Pod has failed, however, Airfow managed to update the state
+        if ti_state in State.finished():
+            # Pod has failed, however, Airflow managed to update the state
             # that means - all code (including dbnd) were executed
             # let just notify the error, so we can show it in summary it
             # we will not send it to databand tracking store
-            task_run.set_task_run_state(
-                TaskRunState.FAILED, track=False, error=task_run_error
-            )
+            dbnd_state = AIRFLOW_TO_DBND_STATE_MAP.get(ti_state, None)
+            task_run.set_task_run_state(dbnd_state, track=False, error=task_run_error)
+
+            if dbnd_state == TaskRunState.FAILED:
+                color = "red"
+            elif dbnd_state == TaskRunState.SUCCESS:
+                color = "cyan"
+            else:
+                color = "yellow"
+
             self.log.info(
                 "%s",
                 task_run.task.ctrl.banner(
-                    "Task %s(%s) - pod %s has failed, airlfow state=Failed!"
-                    % (task_run.task.task_name, task_id, pod_name),
-                    color="red",
+                    "Task %s(%s) - pod %s has failed, airflow state=%s!"
+                    % (task_run.task.task_name, task_id, pod_name, ti_state),
+                    color=color,
                     task_run=task_run,
                 ),
             )
-            return True
-        # we got State.Failed from watcher, but at DB airflow instance in different state
+            return
+
+        # we got State.Failed from watcher, but at DB airflow instance in unfinished state
         # that means the task has failed in the middle
         # (all kind of errors and exit codes)
         task_run_log = error_msg
@@ -503,12 +544,13 @@ class DbndKubernetesScheduler(AirflowKubernetesScheduler):
         )
 
     def _find_pod_failure_reason(
-        self, task_run, pod_name, pod_data,
+        self, pod_name, pod_data,
     ):
+        # type: (str, V1Pod) -> (Optional[PodFailureReason],Optional[str])
         if not pod_data:
             return (
                 PodFailureReason.err_pod_deleted,
-                "Pod %s probably has been deleted (can not be found)" % pod_name,
+                "Pod %s is not found at cluster (deleted/spot/preemptible)" % pod_name,
             )
 
         pod_phase = pod_data.status.phase
@@ -521,9 +563,13 @@ class DbndKubernetesScheduler(AirflowKubernetesScheduler):
             )
             try:
                 pod_ctrl.check_deploy_errors(pod_data)
+            # we handle only known errors
             except KubernetesImageNotFoundError as ex:
                 return PodFailureReason.err_image_pull, str(ex)
+            except KubernetesPodConfigFailure as ex:  # pod config error
+                return PodFailureReason.err_config_error, str(ex)
             except Exception as ex:
+                # we don't want to handle that
                 pass
             return None, None
 
@@ -545,6 +591,7 @@ class DbndKubernetesScheduler(AirflowKubernetesScheduler):
     def _handle_crashed_task_instance(
         self, task_run, task_run_error, failure_reason, session=None
     ):
+        # type: (TaskRun, TaskRunError, PodFailureReason, Optional[Session]) -> None
 
         task_instance = get_airflow_task_instance(task_run, session=session)
         task_instance.task = task_run.task.ctrl.airflow_op
@@ -558,15 +605,13 @@ class DbndKubernetesScheduler(AirflowKubernetesScheduler):
             task_instance.max_tries = retry_count
 
         self.log.info(
-            "Retry %s  task: max_retries=%s, task.retries=%s, current:%s state:%s",
+            "Retry %s task: max_retries=%s, task.retries=%s, current:%s state:%s",
             task_run,
             task_instance.max_tries,
             task_instance.task.retries,
             task_instance._try_number,
             task_instance.state,
         )
-        # retry condition: self.task.retries and self.try_number <= self.max_tries
-        increase_try_number = False
 
         if task_instance.state == State.QUEUED:
             # Special case - no airflow code has been run in the pod at all.
@@ -576,17 +621,55 @@ class DbndKubernetesScheduler(AirflowKubernetesScheduler):
             session.merge(task_instance)
             session.commit()
 
+        # Airflow decide if to retry the task by this condition:
+        # >>> task_instance.task.retries and task_instance.try_number <= task_instance.max_tries
         task_instance.handle_failure(str(task_run_error.exception), session=session)
 
+        # will be logged to help debug why we did or didn't retry the task
+        retry_data = "task.retries={reties}, try_number={try_number}, max_tries={max_tries}.".format(
+            reties=task_instance.task.retries,
+            try_number=task_instance.try_number,
+            max_tries=task_instance.max_tries,
+        )
+
         if task_instance.state == State.UP_FOR_RETRY:
+            self.log.info(
+                "Set the task {task_id} to retry: {data}".format(
+                    task_id=str(task_run), data=retry_data
+                )
+            )
             task_run.set_task_run_state(
                 TaskRunState.UP_FOR_RETRY, track=True, error=task_run_error
             )
         else:
+            self.log.info(
+                "Task {task_id} is not set up to retry: {data}".format(
+                    task_id=str(task_run), data=retry_data
+                )
+            )
             task_run.set_task_run_state(
                 TaskRunState.FAILED, track=True, error=task_run_error
             )
 
-    def get_pod_status(self, pod_name):
-        pod_ctrl = self.kube_dbnd.get_pod_ctrl(name=pod_name)
-        return pod_ctrl.get_pod_status_v1()
+    def handle_zombie_task_instance(self, zombie_task_instance):
+        # type: (SimpleTaskInstance)-> Optional[SubmittedPodState]
+
+        # find a relevant submitted pod based on TaskInstance.key (dag,task,execution_date,try_number)
+        zombie_pod_state = [
+            pod_state
+            for pod_state in self.submitted_pods.values()
+            if pod_state.scheduler_key == zombie_task_instance.key
+        ]
+        if not zombie_pod_state:
+            self.log.info(
+                "Zombie task instance %s is not found at running pods, skipping",
+                zombie_task_instance.key,
+            )
+            return None
+
+        self.log.info(
+            "Processing zombie task instance %s as failed", zombie_task_instance,
+        )
+        zombie_pod_state = zombie_pod_state[0]
+        self._process_pod_failed(zombie_pod_state)
+        return zombie_pod_state
